@echo off
 doskey py27=C:\32\Python27\python.exe $*
 doskey py34=C:\32\Python34\python.exe $*
 doskey py35=C:\32\Python35\python.exe $*
 doskey py36=C:\32\Python36\python.exe $*
 doskey py37=C:\32\Python36\python.exe $*
 doskey py38=C:\32\Python36\python.exe $*
 set Path=C:\msys64\mingw32\bin;%Path%
